<?php

namespace Ardent\ParameterBundle\Controller;

use Ardent\ParameterBundle\Service\ParameterService;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

abstract class BaseController extends AbstractController
{
    public function baseIndex(ParameterService $param, Request $request, array $configuration): Response
    {
        // gather the values
        $data = [];
        foreach ($configuration as $config) {
            $name = $config['name'];
            $data[$name] = $param->get($name, false);
        }

        // build the form
        $formBuilder = $this->createFormBuilder($data);
        foreach ($configuration as $config) {
            $this->configParser($config, $formBuilder);
        }
        $form = $formBuilder->add('Update', SubmitType::class)->getForm();

        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $result = $form->getData();

            foreach ($configuration as $config) {
                $name = $config['name'];
                // Excluding the checkbox type from checking, because it will be null if not checked
                if (null !== $result[$name] || CheckboxType::class == $config['type']) {
                    $param->set($name, $result[$name]);
                }
            }
            $this->addFlash('success', 'Updating parameters successful');

            return $this->redirect($request->getRequestUri());
        }

        return $this->render('@ArdentParameter/form.html.twig', [
            'parameter_form' => $form->createView(),
        ]);
    }

    /**
     * @param                      $config
     * @param FormBuilderInterface $formBuilder
     */
    private function configParser($config, FormBuilderInterface $formBuilder): void
    {
        $type = $config['type'];
        $options = [];
        switch ($type) {
            case ChoiceType::class:
                $choices = array_combine($config['choices'], $config['choices']);

                if (is_callable($choices)) {
                    $options['choices'] = $choices();
                } else {
                    $options['choices'] = $choices;
                }
                break;
            case CheckboxType::class:
                $options['required'] = false;
                break;
            default:
                break;
        }
        $formBuilder->add($config['name'], $type, $options);
    }
}
